/*
 * DS1307.c
 *
 * Created: 13.08.2018 16:39:34
 *  Author: Dirk
 */ 

#include "DS1307.h"
#include "i2cmaster.h"
#include <string.h>

//static rtc_date_t date;
//static rtc_time_t time;
static uint8_t i2c_err = 0;					// Error flag

/*  initialization of DS1307 rtc
*  checks if rtc was running in backup power mode correctly (stored 0x55 at RAM cell 0x08) - returns 0
*  if not - first setup of rtc at 00:00, 1th january 00; weekday=1, no output sqw, osc running - returns 1
*  if i2c communications fails - returns 2
*/

uint8_t DS1307_init (void)
{
	i2c_init();
    
    uint8_t check_running = 0;
	
	i2c_err = i2c_start(DevDS1307 + I2C_WRITE);			// Slave_Addr
	if (i2c_err)
	{
		i2c_stop();
		return 2;
	}
	else
	{
		i2c_write(8);								// Set Addr_Pointer RAM Addr 0x08 (out of rtc-space)
		i2c_rep_start(DevDS1307+I2C_READ);			// repeated start with Slave_Addr read mode
		check_running = i2c_readNak();				// read RAM_check_byte (must be 55h if rtc is running)
	}
	
	if (check_running == 0x55)
	{
		return 0;
	} 
	else
	{
		i2c_err = i2c_start(DevDS1307 + I2C_WRITE);			// Slave_Addr
		if (i2c_err)
		{
			i2c_stop();
			return 2;
		}
		else
		{
			i2c_write(0x00);							// Set Addr_Pointer RAM Addr 0x00 (second register & osc start)
			i2c_write(0x00);							// seconds = 00, osc start
			i2c_write(0x00);							// minutes = 00
			i2c_write(0x00);							// hours = 00, 24h-mode
			i2c_write(0x01);							// day = 1, means monday
			i2c_write(0x01);							// date = 1
			i2c_write(0x01);							// month = 1, means january
			i2c_write(0x00);							// year = 0, means year 2000
			i2c_write(0x00);							// no sqw output (can be configured)
			i2c_write(0x55);							// control-byte rtc running and buffered
			i2c_stop();
		}
		return 1;
	}
}

//__________________________________________________________________________________________________

void rtc_hold (void)
{
	i2c_err = i2c_start(DevDS1307 + I2C_WRITE);
	i2c_write(0x00);							// Set Addr_Pointer RAM Addr 0x00 (second register & osc start)
	i2c_write(0x80);							// seconds = 00, osc stop
	i2c_stop();	
}

void rtc_start (void)
{
	i2c_err = i2c_start(DevDS1307 + I2C_WRITE);
	i2c_write(0x00);							// Set Addr_Pointer RAM Addr 0x00 (second register & osc start)
	i2c_write(0x00);							// seconds = 00, osc start
	i2c_stop();	
}

//************************************************************************************************
// reads rtc values and convert to numbers  
//************************************************************************************************

//uint32_t rtc_read_epoch (void)
//{
	//i2c_start(DevDS1307 + I2C_WRITE);			// Slave_Addr
	//i2c_write(0x00);							// Set Addr_Pointer RAM Addr 0x00
	//i2c_rep_start(DevDS1307 + I2C_READ);
	//uint8_t sec = i2c_readAck();
	//uint8_t min = i2c_readAck();
	//uint8_t hrs = i2c_readNak();
	//i2c_stop();
	//
	//uint32_t epoch = 3600UL * BCD_DEC(hrs) + 60UL * BCD_DEC(min) + BCD_DEC(sec);
	//return epoch;
//}
//
//uint32_t rtc_TimeToEpoch (uint8_t min, uint8_t hrs)
//{
	//uint32_t epoch = 60UL * min + 3600UL * hrs;
	//return epoch;
//}
//
//rtc_time_t rtc_EpochToTime (uint32_t epoch)
//{
////	uint32_t hrs = (epoch / 3600);
////	uint32_t min = (epoch - 3600 * hrs) / 60;
////	uint32_t sec = epoch - (3600 * hrs + 60 * min);
	//time.hours = (uint8_t)(epoch / 3600);
	//time.minutes = (uint8_t)((epoch - 3600UL * time.hours) / 60UL);
	//time.seconds = (uint8_t)(epoch - (3600UL * time.hours + 60UL * time.minutes));
	//return time; 
//}

void rtc_read_time (rtc_time_t *time)
{    
    i2c_start(DevDS1307 + I2C_WRITE);			// Slave_Addr
    i2c_write(0x00);							// Set Addr_Pointer RAM Addr 0x00
    i2c_rep_start(DevDS1307 + I2C_READ);
    time->seconds = BCD_to_DEC(i2c_readAck());
    time->minutes = BCD_to_DEC(i2c_readAck());
    time->hours   = BCD_to_DEC(i2c_readNak());
}



void rtc_read_date (rtc_date_t *date)
{
	i2c_start(DevDS1307 + I2C_WRITE);			// Slave_Addr
	i2c_write(0x03);							// Set Addr_Pointer RAM Addr 0x00
	i2c_rep_start(DevDS1307 + I2C_READ);
	date->weekday = i2c_readAck();
	date->day     = BCD_to_DEC(i2c_readAck());
	date->month   = BCD_to_DEC(i2c_readAck());
	date->year    = BCD_to_DEC(i2c_readNak())+2000;
}


void rtc_set_time (rtc_time_t time)
{
	i2c_err = i2c_start(DevDS1307 + I2C_WRITE);	
	i2c_write(0x00);							// Set Addr_Pointer RAM Addr 0x00 (second register & osc start)
	i2c_write(DEC_to_BCD(time.seconds));			// MSB always 0 = osc start
	i2c_write(DEC_to_BCD(time.minutes));			// minutes set in BCD
	i2c_write(DEC_to_BCD(time.hours));				// hours set in BCD, 24h-format
	i2c_stop();
}

void rtc_set_date (rtc_date_t date)
{
	i2c_err = i2c_start(DevDS1307 + I2C_WRITE);
	i2c_write(0x03);							// Set Addr_Pointer RAM Addr 0x03 (weekday)
	i2c_write(date.weekday);					// 
	i2c_write(DEC_to_BCD(date.day));				// days set in BCD
	i2c_write(DEC_to_BCD(date.month));				// month set in BCD
	i2c_write(DEC_to_BCD((uint8_t)(date.year-2000)));
	i2c_stop();
}


char* rtc_getStrWeekday(char* weekday_str, uint8_t weekday)
{
	switch (weekday)
	{
		case 1:
            weekday_str = strcpy(weekday_str,"So");
            break;
        case 2:
			weekday_str = strcpy(weekday_str, "Mo");
			break;
		case 3:
			weekday_str = strcpy(weekday_str,"Di");
			break;
		case 4:
			weekday_str = strcpy(weekday_str,"Mi");
			break;
		case 5:
			weekday_str = strcpy(weekday_str,"Do");
			break;
		case 6:
			weekday_str = strcpy(weekday_str,"Fr");
			break;
		case 7:
			weekday_str = strcpy(weekday_str,"Sa");
			break;
		default:
			weekday_str = strcpy(weekday_str,"Er");
			break;	
	}
	return weekday_str;
} 


// conversation BCD to DEC

uint8_t BCD_to_DEC(uint8_t val)
{
	return (val/16*10) + (val%16);
}

uint8_t DEC_to_BCD(uint8_t val)
{
	return (val/10*16) + (val%10);
}


// calculates the weekday (1=Sunday, 7=Saturday) from date
//uint8_t calc_weekday(uint8_t day, uint8_t month, uint8_t year)
//{
	//return (((day += month < 3 ? year-- : year - 2, 23*month/9 + day + 4 + year/4- year/100 + year/400)%7)+1);
//}