/*
 * pir.c
 *
 */ 

#include <avr/io.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#include "pir.h"

// we assume that we have presence when device are released from reset
static volatile bool presence;
static volatile uint16_t counter;    // since a tick is 2 seconds, multiply by 30 to get minutes

void pir_init(bool presence_start_condition)
{
    PIR_PORT.DIRCLR = PIR_PIN_bm;
    
    if (presence_start_condition)   // assume presence at initialization
    {
        counter = (PIR_TIMEOUT*30-1);
        presence = true;
    } 
    else                            // assume no presence at initialization
    {
        counter = 0;
        presence = false;
    }
    
    PIR_TIMER.CNT = 0;                          // reset timer   
    PIR_TIMER.PER = (F_CPU*2)/1024 - 1;         // That should give an interrupt every two seconds
    PIR_TIMER.INTCTRLA = TC_OVFINTLVL_LO_gc;    // raise low priority interrupt on overflow
    PIR_TIMER.CTRLA = TC_CLKSEL_DIV1024_gc;
}

bool pir_presence(void)
{
    return presence;
}

void pir_poll(void)
{
    if ((PIR_PORT.IN & PIR_PIN_bm) > 0)   // PIR sensor detects someone
    {
        PIR_TIMER.CTRLA = TC_CLKSEL_OFF_gc;         // stop timer to avoid interfering timer interrupts
        counter = (PIR_TIMEOUT*30-1);               // reload soft counter
        PIR_TIMER.CNT = 0;                          // reset timer
        presence = true;                    
        PIR_TIMER.CTRLA = TC_CLKSEL_DIV1024_gc;     // start timer again
    } 
}

ISR(PIR_OVF_vect)                         // every two seconds
{
    if (counter > 0)
    {
        counter --;
    }
    if (counter == 0)
    {
        presence = false;
    }     
}