/*
 * ws2812.c
 *
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "ws2812.h"


uint8_t color_data[WS2812_DATALEN];


void WS2812_init(void)
{
    WS2812_PORT.OUTCLR=WS2812_PIN_bm;  // initialize PIN with low
    WS2812_PORT.DIRSET=WS2812_PIN_bm;  // pin is output
}

static inline void WS2812_reset(void)
{
    WS2812_PORT.OUTCLR=WS2812_PIN_bm;  // make sure that pin is low for 50 us
    _delay_us(50);
}


// _delay_loop_1 = 3 cycles a 31.25 ns (at 32 MHz) up to 256 cycles (0=256)
// _delay_loop_2 = 4 cycles a 31.25 ns up to 65536

static inline void WS2812_sendOne(void)
{
    // 900 (+-150) ns high = 28.8 cycles (+- 4 cycles)
    // 350 (+-150) ns low  = 11.2 cycles
    
    WS2812_PORT.OUTSET=WS2812_PIN_bm;      
    _delay_loop_2(6);                      
    WS2812_PORT.OUTCLR=WS2812_PIN_bm;      
    // loop takes already enough time for the 350 ns
}

static inline void WS2812_sendZero(void)
{
    // 350 (+-150) ns high
    // 900 (+-150) ns low
    
    WS2812_PORT.OUTSET=WS2812_PIN_bm;
    _delay_loop_2(1);
    asm volatile ("nop"); asm volatile ("nop");
    WS2812_PORT.OUTCLR=WS2812_PIN_bm;
    _delay_loop_2(4);
    asm volatile ("nop"); asm volatile ("nop");
}

static inline void WS2812_send_byte(uint8_t byte)
{    
    uint8_t cnt=8;
    while(cnt--)
    {
        if(byte & 0x80) WS2812_sendOne();
        else WS2812_sendZero();
        byte<<=1;
    }
}

void WS2812_update(void)
{
    uint8_t i = 0;
    cli();
    WS2812_reset();
    for (i=0; i < (WS2812_DATALEN); i++)
    {
        WS2812_send_byte(color_data[i]);
    }
    sei();    
}

void WS2812_set_color(uint8_t r, uint8_t g, uint8_t b, uint8_t index)
{
    if (index >= WS2812_LEDCNT)     // check for valid led number
    {
        return;
    }
    color_data[index*3+0] = g;
    color_data[index*3+1] = r;
    color_data[index*3+2] = b;    
}